unit UfrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Contnrs, StdCtrls, Grids, ComCtrls, Buttons, System.ImageList,
  Vcl.ImgList, Vcl.ToolWin, Progs;

type
  TfrmMain = class(TForm)
    gboxProgs: TGroupBox;
    gboxSteps: TGroupBox;
    dgProgs: TDrawGrid;
    btnInsProg: TSpeedButton;
    btnDelProg: TSpeedButton;
    btnProgMoveUp: TSpeedButton;
    btnProgMoveDown: TSpeedButton;
    dgSteps: TDrawGrid;
    btnDelStep: TSpeedButton;
    btnInsStep: TSpeedButton;
    btnStepMoveUp: TSpeedButton;
    btnStepMoveDown: TSpeedButton;
    sBar: TStatusBar;
    tbMain: TToolBar;
    btnFileNew: TToolButton;
    btnFileOpen: TToolButton;
    btnFileSave: TToolButton;
    ToolButton4: TToolButton;
    btnFileSaveAs: TToolButton;
    ToolButton5: TToolButton;
    btnFileOptions: TToolButton;
    btnGenerate: TToolButton;
    ToolButton6: TToolButton;
    btnAbout: TToolButton;
    ToolButton7: TToolButton;
    btnExit: TToolButton;
    imMain: TImageList;
    gbSrc: TGroupBox;
    memSrc: TMemo;
    btnSaveHeader: TToolButton;
    btnPreview: TToolButton;
    ToolButton2: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure dgProgsDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure dgProgsGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure dgProgsSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure dgStepsDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure dgStepsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnDelStepClick(Sender: TObject);
    procedure btnInsStepClick(Sender: TObject);
    procedure btnStepMoveUpClick(Sender: TObject);
    procedure btnStepMoveDownClick(Sender: TObject);
    procedure btnInsProgClick(Sender: TObject);
    procedure btnDelProgClick(Sender: TObject);
    procedure btnProgMoveUpClick(Sender: TObject);
    procedure btnProgMoveDownClick(Sender: TObject);
    procedure dgProgsSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure btnFileSaveClick(Sender: TObject);
    procedure btnFileOptionsClick(Sender: TObject);
    procedure btnFileNewClick(Sender: TObject);
    procedure btnFileOpenClick(Sender: TObject);
    procedure btnGenerateClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnFileSaveAsClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure dgProgsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure dgProgsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnSaveHeaderClick(Sender: TObject);
    procedure btnPreviewClick(Sender: TObject);
  private
    { Private declarations }
  public
    FProgList: TProgList;
    FModified: Boolean;
    FFileName: String;          //   
    procedure DisplayHint(Sender: TObject);
    procedure ShowFileName(const AFileName: String);
    function FileDoSaveAs: Boolean;
    function FileDoSave: Boolean;
    procedure FileDoSaveHeader;
    function CheckFileSave: Boolean;
    procedure SetModified(Value: Boolean);
    procedure SetColWidths;
    procedure FileDoNew;
    procedure FileDoLoad(const AFileName: String);
    procedure FileDoOpen;
    procedure SelectCell(AGrid: TDrawGrid; ACol, ARow: Integer);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses
  UfrmFileOptions, UfrmGenSrcOptions, UfrmAbout, UfrmPreview;

resourcestring
  rsUntitled = ' ';
  rsFilter = '   (*.lel)|*.lel';
  rsDefaultExt = 'lel';

  rsHeaderFilter = '  C++ (*.h)|*.h';
  rsHeaderDefaultName = 'lel_prog.h';

  rsModified = '';
  rsSaveChange = '    "%s"?';
  rsShowFileName = '%s - %s';
  rsDefComment = '      (.   - 232 ).';
  rsNumberCap = 'N';
  rsProgNameCap = '';
  rsGridStepLeft = '<=';
  rsGridStepRight = '=>';
  rsGridStepNone = '==';
  rsGridStepFontName = 'Courier New';
  rsMinStepReached = ' . .  : %d.   .';
  rsMaxStepReached = ' . .  : %d.    .';
  rsMinProgReached = ' . . : %d.   .';
  rsMaxProgReached = ' . . : %d.    .';

const
  TXT_CDRAW_FLAGS = DT_CENTER or DT_VCENTER or DT_SINGLELINE;
  TXT_LDRAW_FLAGS = DT_LEFT or DT_VCENTER or DT_SINGLELINE;
  RDIR_COLOR = clRed;
  LDIR_COLOR = clBlue;
  SEL_COLOR = clYellow;

{ TfrmMain }

procedure TfrmMain.DisplayHint(Sender: TObject);
begin
  if Length(GetLongHint(Application.Hint)) > 0 then
  begin
    sBar.SimplePanel:= True;
    sBar.SimpleText:= GetLongHint(Application.Hint);
  end else
    sBar.SimplePanel:= False;
end;

procedure TfrmMain.ShowFileName(const AFileName: String);
begin
  FFileName:= AFileName;
  if FFileName <> '' then
    Caption:= Application.Title + ' - ' + ExtractFileName(FFileName)
  else
    Caption:= Application.Title;
  if FFileName = '' then
  begin
    sBar.Panels[1].Text:= '';
  end else
  begin
    sBar.Panels[1].Text:= FFileName;
  end;
end;

procedure TfrmMain.btnPreviewClick(Sender: TObject);
begin
//  FileDoLoad(ExtractFilePath(Application.ExeName) + 'lel_prog.lel');
  Preview(FProgList);
end;

procedure TfrmMain.SetModified(Value: Boolean);
begin
  FModified:= Value;
  if FModified then
    sBar.Panels[0].Text:= rsModified
  else
    sBar.Panels[0].Text:= '';
end;

function TfrmMain.FileDoSaveAs: Boolean;
begin
  Result:= False;
  with TSaveDialog.Create(Self) do
  begin
    Filter:= rsFilter;
    DefaultExt:= rsDefaultExt;
    Options:= [ofHideReadOnly, ofOverwritePrompt, ofEnableSizing];
    if FFileName = rsUntitled then
      InitialDir:= ExtractFilePath(Application.ExeName);
    FileName:= ExtractFileName(FFileName);
    if not Execute then Exit;
    FFileName:= FileName;
    FProgList.SaveToFile(FFileName);
    Caption:= Format(rsShowFileName, [Application.Title, ExtractFileName(FFileName)]);
    SetModified(False);
    Result:= True;
  end;
end;

function TfrmMain.FileDoSave: Boolean;
begin
  if FFileName = rsUntitled then
    Result:= FileDoSaveAs
  else
  begin
    FProgList.SaveToFile(FFileName);
    SetModified(False);
    Result:= True;
  end;
end;

procedure TfrmMain.FileDoSaveHeader;
begin
  with TSaveDialog.Create(Self) do
  begin
    Filter:= rsHeaderFilter;
    DefaultExt:= 'h';
    Options:= [ofHideReadOnly, ofOverwritePrompt, ofEnableSizing];
    if FFileName = rsUntitled then
      InitialDir:= ExtractFilePath(Application.ExeName)
    else
      InitialDir:= ExtractFilePath(FFileName);
    FileName:= rsHeaderDefaultName;
    if not Execute then Exit;
    try
      memSrc.Lines.SaveToFile(FileName);
    except
      raise;
    end;
  end;
end;

function TfrmMain.CheckFileSave: Boolean;
begin
  Result:= False;
  if not FModified then
  begin
    Result:= True;
    Exit;
  end;
  case Application.MessageBox(PChar(Format(rsSaveChange, [FFileName])),
     PChar(Application.Title), MB_ICONQUESTION or MB_YESNOCANCEL) of
    IDYES   : Result:= FileDoSave;
    IDNO    : Result:= True;
    IDCANCEL: Result:= False;
  end;
end;

procedure TfrmMain.SelectCell(AGrid: TDrawGrid; ACol, ARow: Integer);
var
  grRect: TGridRect;
begin
  grRect.Left:= ACol;
  grRect.Top:= ARow;
  grRect.Right:= ACol;
  grRect.Bottom:= ARow;
  AGrid.Selection:= grRect;
end;

procedure TfrmMain.FileDoNew;
var
  nYear, nMonth, nDay: Word;
begin
  if not CheckFileSave then Exit;
  dgProgs.OnDrawCell:= nil;
  dgSteps.OnDrawCell:= nil;
  try
    FProgList.Clear;
    FProgList.Add(TProg.Create(DEF_STEP_COUNT, Format(rsDefProgName, [FProgList.Count + 1])));
    dgProgs.RowCount:= FProgList.Count + 1;
    SelectCell(dgProgs, 1, 1);
    dgSteps.RowCount:= FProgList[dgProgs.Row - 1].StepCount + 1;
    DecodeDate(Now, nYear, nMonth, nDay);
    FillChar(LelFileHeader, SizeOf(TLelFileHeader), 0);
    with LelFileHeader do
    begin
      Signature:= FILE_SIGNATURE;
      Version:= 0;
      Reserved1:= 0;
      Day:= nDay;
      Month:= nMonth;
      Year:= nYear;
      Count:= FProgList.Count;
      FillChar(Reserved2, SizeOf(Reserved2), Ord('A'));
      Comment:= AnsiString(rsDefComment);
    end;
    ShowFileName(rsUntitled);
    SetModified(False);
  finally
    dgProgs.OnDrawCell:= dgProgsDrawCell;
    dgSteps.OnDrawCell:= dgStepsDrawCell;
    dgProgs.Invalidate;
    dgSteps.Invalidate;
  end;
end;

procedure TfrmMain.FileDoLoad(const AFileName: String);
begin
  dgProgs.OnDrawCell:= nil;
  dgSteps.OnDrawCell:= nil;
  try
    if FProgList.LoadFromFile(AFileName) then
    begin
      dgProgs.RowCount:= FProgList.Count + 1;
      SelectCell(dgProgs, 1, 1);
      dgSteps.RowCount:= FProgList[dgProgs.Row - 1].StepCount + 1;
      ShowFileName(AFileName);
      SetModified(False);
    end else
      FileDoNew;
  finally
    dgProgs.OnDrawCell:= dgProgsDrawCell;
    dgSteps.OnDrawCell:= dgStepsDrawCell;
    dgProgs.Invalidate;
    dgSteps.Invalidate;
  end;
end;

procedure TfrmMain.FileDoOpen;
begin
  if not CheckFileSave then Exit;
  with TOpenDialog.Create(Self) do
  begin
    Filter:= rsFilter;
    DefaultExt:= rsDefaultExt;
    Options:= [ofHideReadOnly, ofEnableSizing];
    if FFileName = rsUntitled then
      InitialDir:= ExtractFilePath(Application.ExeName);
    FileName:= FFileName;
    if not Execute then Exit;
    FileDoLoad(FileName);
  end;
end;

procedure TfrmMain.SetColWidths;
var
  w: Integer;
begin
  w:= dgProgs.Canvas.TextWidth('W');
  dgProgs.ColWidths[0]:= w * 3;
  dgProgs.ColWidths[1]:= w * 14;
  dgSteps.ColWidths[0]:= w * 3;
  dgSteps.ColWidths[1]:= w * 3;
  dgSteps.ColWidths[2]:= w * 3;
  dgSteps.ColWidths[3]:= w * 3;
  dgSteps.ColWidths[4]:= w * 3;
  FProgList:= TProgList.Create;
  FileDoNew;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:= CheckFileSave;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Application.OnHint:= DisplayHint;
  SetColWidths;
  FProgList:= TProgList.Create;
  FileDoNew;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FProgList.Free;
end;

procedure TfrmMain.dgProgsDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  S: String;
  R: TRect;
begin
  with (Sender as TDrawGrid) do
  begin
    if ARow = 0 then
    begin
      if ACol = 0 then
        S:= rsNumberCap
      else
        S:= rsProgNameCap;
      DrawText(Canvas.Handle, PChar(S), Length(S), Rect, TXT_CDRAW_FLAGS);
    end else
    begin
      if ACol = 0 then
      begin
        S:= IntToStr(ARow);
        DrawText(Canvas.Handle, PChar(S), Length(S), Rect, TXT_CDRAW_FLAGS);
      end else
      begin
        S:= FProgList[ARow - 1].Hint;
        R:= Rect;
        InflateRect(R, -2, 0);
        DrawText(Canvas.Handle, PChar(S), Length(S), R, TXT_LDRAW_FLAGS);
      end;
    end;
  end;
end;

procedure TfrmMain.dgProgsGetEditText(Sender: TObject; ACol, ARow: Integer;
  var Value: String);
begin
  Value:= FProgList[ARow - 1].Hint;
end;

procedure TfrmMain.dgProgsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if dgProgs.EditorMode and (Key = VK_ESCAPE) then
    dgProgs.EditorMode:= False;
end;

procedure TfrmMain.dgProgsMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  ACol, ARow: Integer;
begin
  with (Sender as TDrawGrid) do
  begin
    MouseToCell(X, Y, ACol, ARow);
    if (ACol < 0) or (ARow < 0) or (ACol < FixedCols) or (ARow < FixedRows)then
      if EditorMode then
        EditorMode:= False;
  end;
end;

procedure TfrmMain.dgProgsSetEditText(Sender: TObject; ACol, ARow: Integer;
  const Value: String);
begin
  FProgList[ARow - 1].Hint:= AnsiString(Copy(Value, 1, MAX_HINT_LEN));
  SetModified(True);
end;

procedure TfrmMain.dgStepsDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  S: String;
  B: Byte;
begin
  with (Sender as TDrawGrid) do
  begin
    if (gdSelected in State) then
    begin
      Canvas.Brush.Color:= SEL_COLOR;
      Canvas.FillRect(Rect);
    end;
    if ARow = 0 then
    begin
      if ACol = 0 then
        S:= rsNumberCap
      else
        S:= IntToStr(ACol);
      DrawText(Canvas.Handle, PChar(S), Length(S), Rect, TXT_CDRAW_FLAGS);
    end else
    begin
      if ACol = 0 then
      begin
        S:= IntToStr(ARow);
        DrawText(Canvas.Handle, PChar(S), Length(S), Rect, TXT_CDRAW_FLAGS);
      end else
      begin
        B:= FProgList[dgProgs.Row - 1].Steps[ARow - 1];
        if TstBit(B, 8 - ACol) then   // 
        begin
          if TstBit(B, 4 - ACol) then // 
          begin
            S:= rsGridStepLeft;
            Canvas.Font.Color:= LDIR_COLOR;
          end else
          begin
            S:= rsGridStepRight;
            Canvas.Font.Color:= RDIR_COLOR;
          end;
        end else
        begin
          S:= rsGridStepNone;
          Canvas.Font.Color:= TColor($DFDFDF);
        end;
        Canvas.Font.Name:= rsGridStepFontName;
        Canvas.Font.Style:= [fsBold];
        DrawText(Canvas.Handle, PChar(S), Length(S), Rect, TXT_CDRAW_FLAGS);
        Canvas.Font:= Font;
      end;
    end;
  end;
end;

procedure TfrmMain.dgStepsMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  ACol, ARow: Integer;
  B: Byte;
begin
  if (ssCtrl in Shift) and (ssLeft in Shift) then
  begin
    dgSteps.MouseToCell(X, Y, ACol, ARow);
    if (ACol > 0) and (ARow > 0) then
      begin
      B:= FProgList[dgProgs.Row - 1].Steps[ARow - 1];
      if TstBit(B, 8 - ACol) then    //  
      begin
        if TstBit(B, 4 - ACol) then  //   
          B:= RstBit(B, 8 - ACol)    // 
        else                         //   
          B:= SetBit(B, 4 - ACol)    //    
      end else                       //  - 
      begin
        B:= SetBit(B, 8 - ACol);     // 
        B:= RstBit(B, 4 - ACol)      //     
      end;
      FProgList[dgProgs.Row - 1].Steps[ARow - 1]:= B;
      dgSteps.Invalidate;
      SetModified(True);
    end;
  end;
end;

procedure TfrmMain.btnDelStepClick(Sender: TObject);
begin
  if FProgList[dgProgs.Row - 1].StepCount < 2 then
  begin
    Application.MessageBox(PChar(Format(rsMinStepReached, [1])),
      PChar(Application.Title), MB_ICONWARNING or MB_OK);
    Exit;
  end;
  dgSteps.OnDrawCell:= nil;
  FProgList[dgProgs.Row - 1].Delete(dgSteps.Row - 1);
  dgSteps.RowCount:= FProgList[dgProgs.Row - 1].StepCount + 1;
  dgSteps.OnDrawCell:= dgStepsDrawCell;
  dgSteps.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.btnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.btnInsStepClick(Sender: TObject);
begin
  if FProgList[dgProgs.Row - 1].StepCount > (MAX_STEP_COUNT - 1) then
  begin
    Application.MessageBox(PChar(Format(rsMaxStepReached, [MAX_STEP_COUNT])),
      PChar(Application.Title), MB_ICONWARNING or MB_OK);
    Exit;
  end;
  dgSteps.OnDrawCell:= nil;
  FProgList[dgProgs.Row - 1].Insert(dgSteps.Row - 1);
  dgSteps.RowCount:= FProgList[dgProgs.Row - 1].StepCount + 1;
  dgSteps.OnDrawCell:= dgStepsDrawCell;
  dgSteps.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.btnStepMoveUpClick(Sender: TObject);
begin
  if dgSteps.Row < 2 then
    Exit;
  FProgList[dgProgs.Row - 1].Exchange(dgSteps.Row - 1, dgSteps.Row - 2);
  SelectCell(dgSteps, dgSteps.Col, dgSteps.Row - 1);
  dgSteps.Invalidate;
end;

procedure TfrmMain.btnSaveHeaderClick(Sender: TObject);
begin
  FileDoSaveHeader;
end;

procedure TfrmMain.btnStepMoveDownClick(Sender: TObject);
begin
  if dgSteps.Row > (dgSteps.RowCount - 2) then
    Exit;
  FProgList[dgProgs.Row - 1].Exchange(dgSteps.Row - 1, dgSteps.Row);
  SelectCell(dgSteps, dgSteps.Col, dgSteps.Row + 1);
  dgSteps.Invalidate;
end;

procedure TfrmMain.btnInsProgClick(Sender: TObject);
begin
  if FProgList.Count > (MAX_PROG_COUNT - 1) then
  begin
    Application.MessageBox(PChar(Format(rsMaxProgReached, [MAX_PROG_COUNT])),
      PChar(Application.Title), MB_ICONWARNING or MB_OK);
    Exit;
  end;
  dgProgs.OnDrawCell:= nil;
  FProgList.Insert(dgProgs.Row - 1, TProg.Create(DEF_STEP_COUNT, Format(rsDefProgName, [FProgList.Count + 1])));
  dgProgs.RowCount:= FProgList.Count + 1;
  dgProgs.OnDrawCell:= dgProgsDrawCell;
  dgProgs.Invalidate;
  dgSteps.OnDrawCell:= nil;
  dgSteps.RowCount:= FProgList[dgProgs.Row - 1].StepCount + 1;
  dgSteps.OnDrawCell:= dgStepsDrawCell;
  dgSteps.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  AboutDlgExec;
end;

procedure TfrmMain.btnDelProgClick(Sender: TObject);
begin
  if FProgList.Count < 2 then
  begin
    Application.MessageBox(PChar(Format(rsMinProgReached, [1])),
      PChar(Application.Title), MB_ICONWARNING or MB_OK);
    Exit;
  end;
  dgProgs.OnDrawCell:= nil;
  FProgList.Delete(dgProgs.Row - 1);
  dgProgs.RowCount:= FProgList.Count + 1;
  dgProgs.OnDrawCell:= dgProgsDrawCell;
  dgProgs.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.btnProgMoveUpClick(Sender: TObject);
begin
  if dgProgs.Row < 2 then
    Exit;
  FProgList.Exchange(dgProgs.Row - 1, dgProgs.Row - 2);
  SelectCell(dgProgs, dgProgs.Col, dgProgs.Row - 1);
  dgProgs.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.btnProgMoveDownClick(Sender: TObject);
begin
  if dgProgs.Row > (dgProgs.RowCount - 2) then
    Exit;
  FProgList.Exchange(dgProgs.Row - 1, dgProgs.Row);
  SelectCell(dgProgs, dgProgs.Col, dgProgs.Row + 1);
  dgProgs.Invalidate;
  SetModified(True);
end;

procedure TfrmMain.dgProgsSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  dgSteps.OnDrawCell:= nil;
  dgSteps.RowCount:= FProgList[ARow - 1].StepCount + 1;
  dgSteps.OnDrawCell:= dgStepsDrawCell;
  dgSteps.Invalidate;
end;

procedure TfrmMain.btnFileNewClick(Sender: TObject);
begin
  FileDoNew;
end;

procedure TfrmMain.btnFileOpenClick(Sender: TObject);
begin
  FileDoOpen;
end;

procedure TfrmMain.btnFileSaveAsClick(Sender: TObject);
begin
  FileDoSaveAs;
end;

procedure TfrmMain.btnFileSaveClick(Sender: TObject);
begin
  FileDoSave;
end;

procedure TfrmMain.btnGenerateClick(Sender: TObject);
begin
  if EditGenerateOptions(FProgList) then
    FProgList.GenSrc(memSrc.Lines);
end;

procedure TfrmMain.btnFileOptionsClick(Sender: TObject);
begin
  if EditFileOptions then
    SetModified(True);
end;

end.
 